#!/bin/sh
# Copyright (c) 2008 TeamF1, Inc.
# All rights reserved.
#
# This script captures the state of a VPN router to be downloaded
# and sent to TeamF1 for field support.
#

# Validate Session - this is only valid in recovery mode
[ "$RECOVERY_MODE" = "1" ] || exit 0

#
# HTTP content type
echo "Content-type: application/gzip"
echo "Content-disposition: attachment; filename=`hostname`dbg.tgz"
echo

logdir="/var"
tmpfile=/tmp/dbgerr.log
#
# create a temp log directory
echo mkdir -p $logdir/dbg > $tmpfile 2> $tmpfile

mkdir -p $logdir/dbg > $tmpfile 2> $tmpfile

# Run each command and log to a tmp file. The tmpfile will be
# put in the tar file so that we can examine the execution
# of these commands themselves


# This is the dir where temp log files are created
cd $logdir/dbg/

# Run a bunch of commands to capture the state of the box

# Process state
 ps                         > ps.log 2>> $tmpfile

# network state
 ifconfig -a                > ifconfig.log 2>> $tmpfile
 ip route show table all    > route.log 2>> $tmpfile

 cat /proc/sys/net/ipv4/netfilter/ip_conntrack_count  > conntrack.log 2>> $tmpfile
 cat /proc/net/stat/ip_conntrack                     >> conntrack.log 2>> $tmpfile
 cat /proc/net/ip_conntrack | wc -l                  >> conntrack.log 2>> $tmpfile
 cat /proc/net/ip_conntrack                          >> conntrack.log 2>> $tmpfile

 cat /proc/net/stat/arp_cache  > arp.log 2>> $tmpfile
 cat /proc/net/arp            >> arp.log 2>> $tmpfile

 cat /proc/mounts              > mounts.log 2>> $tmpfile
 cat /proc/cmdline             > cmdline.log 2>> $tmpfile


 tagNCat() { [ -f $1 ] && echo "[$1]" && cat $1 2>> $tmpfile && echo; }

 tagNCat /proc/cpuinfo       > hwinfo.log
 tagNCat /proc/octeon_info  >> hwinfo.log
 tagNCat /proc/mtd          >> hwinfo.log
 tagNCat /proc/pci          >> hwinfo.log
 tagNCat /proc/misc         >> hwinfo.log
 tagNCat /proc/interrupts   >> hwinfo.log

 tagNCat /proc/meminfo       > meminfo.log
 tagNCat /proc/ioports      >> meminfo.log
 tagNCat /proc/iomem        >> meminfo.log
 tagNCat /proc/dma          >> meminfo.log

 # Dump a leading portion of each partition
 if [ -e /proc/mtd ] ; then
  for i in $(cut -f1 -d: /proc/mtd | tail -n +2) ; do
   dd if=/dev/$i of=${i}_head.bin bs=512 count=1 > /dev/null 2>> $tmpfile
  done
 fi

# Tar up all the files we need for debugging
# errors from tar will be ignored..
tar czf - /var $tmpfile

#cleanup
rm -fr $logdir/dbg $tmpfile

# Done
exit

